#include "RxThresholdBasedOnConfigRegisterConfigUpdater.h"
#include "ConfigUpdaterManager.h"
#include "../SctApiAccessException.h"
#include "../SctApiCall.h"

#include "Sct_SctApi/Sct_SctApi.hh"
#include "Sct/SctParameters.h"
#include "Sct/SctNames.h"
#include "SctData/RxThresholdBasedOnConfigRegisterTestResult.h"
#include "CommonWithDsp/ABCD/ABCDscans.h"
#include "SctApi/utility.h"

using namespace Sct;
using namespace SctData;

namespace SctCalibrationController {

bool RxThresholdBasedOnConfigRegisterConfigUpdater::inMap = ConfigUpdaterManager::instance().setUpdater("SctData::RxThresholdBasedOnConfigRegisterTestResult", shared_ptr<ConfigUpdater> (new RxThresholdBasedOnConfigRegisterConfigUpdater()));

  void RxThresholdBasedOnConfigRegisterConfigUpdater::updateUsing(const int REGISTER, const int threshold, const unsigned int linkNumber, Sct_SctApi_T_SctApi& api, const unsigned long mid, const std::string & name) {
    if (RxThresholdBasedOnConfigRegisterTestResult::StreamResult::isValidRxThresh(threshold)) {
      
      unsigned partition, crate, rod, channel;
      SctApi::Utility::getpcrc(mid, partition, crate, rod, channel);
  
      APICALL(&api, modifyBOCParam(&st, partition, crate, rod, channel, REGISTER, threshold ), "RxDelayConfigUpdater failed to set delay") ;
      std::ostringstream os;
      os << "Setting Register " << REGISTER << " for link " << linkNumber << " of module " << mid << " named " << name << " to " << threshold << endl;
      os << "Partition=" << partition << "; Crate=" << crate << "; Rod=" << rod << "; Chan=" << channel << endl;
      SctNames::Mrs() << "CC_UPDATE" << MRS_INFORMATION << MRS_TEXT(os.str()) << ENDM;
    } else {
      std::ostringstream os;
      os << "Warning: was unable to set the RxThreshold for link "<<linkNumber<<" of module " << mid << " as the test was not able to determine a sensible value for it!";
      SctNames::Mrs() << "CC_UPDATE" << MRS_WARNING << MRS_TEXT(os.str()) << ENDM;
    };
  };

void RxThresholdBasedOnConfigRegisterConfigUpdater::update(const TestResult& testresult, Sct_SctApi_T_SctApi& api) const {
    const RxThresholdBasedOnConfigRegisterTestResult& t = dynamic_cast<const RxThresholdBasedOnConfigRegisterTestResult&> (testresult);

    unsigned long mid = getMID(t, api);
    SctNames::Mrs() << "CC_UPDATE" << MRS_TEXT("CalibrationController updating RxThresholdBasedOnConfigRegister") << MRS_PARAM<long>("ModuleID", mid) << MRS_DIAGNOSTIC << ENDM;

    const int link0Thresh = t.getStreamResult(0).bestRxThresh();
    const int link1Thresh = t.getStreamResult(1).bestRxThresh();
    
    const std::string & name = t.getModuleName();

    this->updateUsing(ST_RX_THRESHOLD0, link0Thresh, 0, api, mid, name);
    this->updateUsing(ST_RX_THRESHOLD1, link1Thresh, 1, api, mid, name);    
}
    
}
