#ifndef DOUBLETRIGGERNOISETEST_H
#define DOUBLETRIGGERNOISETEST_H

#include <cmath>  // For fabs

#include "../ipc/Sct_CalibrationController.hh"
#include "../CalibrationControllerImpl.h"
#include "../ipc/ScanRequest.h"
#include "../SctApiAccessException.h"
#include "../SctApiCall.h"
#include "Sct/LogicErrors.h"

#include <CommonWithDsp/ABCD/ABCDscans.h>

using namespace Sct;
using namespace SctCalibrationController::Ipc;
namespace SctCalibrationController {
namespace Scripts {

class DoubleTriggerNoiseTest  : public TestRequestImpl {
public:
    static DoubleTriggerNoiseTest& instance() {
      static DoubleTriggerNoiseTest dtnt;
      return dtnt;
    }

    virtual void startTest(Sct_SctApi_T_SctApi& api, string& name, unsigned short& testVariable, unsigned long& nScans, vector<double>& testPoints) {	
	this->api = &api;

        // Compression mode X1X
        APICALL(&api, modifyABCDVar(&st, ST_COMPRESSION, 1), "DoubleTriggerNoiseTest::startTest")

        // Edge detect off
        APICALL(&api, modifyABCDVar(&st, ST_EDGE_DETECT, 0), "DoubleTriggerNoiseTest::startTest")

        // No masked channels
        APICALL(&api, modifyABCDVar(&st, ST_SEND_MASK, 0), "DoubleTriggerNoiseTest::startTest")

        // 1 fC threshold
        APICALL(&api, modifyABCDVar(&st, ST_QTHR, 0.9), "DoubleTriggerNoiseTest::startTest")

	name = "DoubleTriggerNoiseTest";
	testVariable = ST_ELAPSED;
	nScans = 1;
	testPoints.push_back(0.);
    }


    virtual ScanRequest* getNextScan(unsigned long index) {
        ScanLibrary& sl = CalibrationControllerImpl::instance().getScanLibrary();
        ScanRequestImpl& request = sl.defaultScan();

	// Create scan
        Sct_SctApi_T_Scan* s = 0;
        APIRETCALL(api, s, createScan(&st), "DoubleTriggerNoiseTest::getNextScan")
        request.setScan(s);

        APICALL(s, setNTrigs(&st, 100000), "Failed to set triggers")

        // This doesn't work with group distribution which is the default otherwise
	APICALL(s, setOption(&st, Sct_SctApi_T_Scan_ScanOptions_distSlave, 0), "Failed to set distSlave option")

        // Only capture every 2nd trigger
	APICALL(s, setOption(&st, Sct_SctApi_T_Scan_ScanOptions_nth, 2), "Failed to set nth option")

        // Capture the second of the two triggers (index 0)
	APICALL(s, setOption(&st, Sct_SctApi_T_Scan_ScanOptions_nth_rem, 0), "Failed to set nth_rem option")

	// Adjust trigger
        Sct_SctApi_T_Trigger* t = 0;
        APIRETCALL(s, t, getTrigger1(&st), "Failed to get trigger1")

	int startDelay = 120;
	int delayStep = 1;
	int stopDelay = 160;

        APICALL(t, doubleL1A(&st, startDelay), "Failed to set trigger type")

	// Increment delay between triggers
	APICALL(t, setCommIncr(&st, 1, delayStep), "DoubleTriggerNoiseTest::getNextScan")

	// Configure scan
        APICALL(s, configure(&st, ST_ELAPSED, startDelay, stopDelay, delayStep), "Failed to set Scan information")

	return &request;
    }

    virtual bool canFeedback() {
	return false;
    }

    virtual void endTest() {
	api = 0;
    }

private:
    DoubleTriggerNoiseTest() {}
    Sct_SctApi_T_SctApi* api;
};
    
}
}

#endif //DOUBLETRIGGERNOISETEST_H
