#ifndef SCTCALIBRATIONCONTROLLER_RXTHRESHOLDBASEDONCONFIGREGISTERTEST_H
#define SCTCALIBRATIONCONTROLLER_RXTHRESHOLDBASEDONCONFIGREGISTERTEST_H

#include "../ipc/TestRequest.h"
#include "../ipc/CalibrationController.h"
#include "../ipc/ScanLibrary.h"
#include "../ipc/Sct_CalibrationController.hh"

#include "../SctApiAccessException.h"
#include "../SctApiCall.h"
#include "../CalibrationControllerImpl.h"
#include "Sct/LogicErrors.h"
#include "SctApi/extraScans.h"

#include <CommonWithDsp/ABCD/ABCDscans.h>
#include <iostream>
#include <vector>

using namespace Sct;
using namespace std;
using namespace SctCalibrationController::Ipc;
using namespace SctCalibrationController;

namespace SctCalibrationController {
namespace Scripts {
    
class RxThresholdBasedOnConfigRegisterTest : public TestRequestImpl {
public:	
    static RxThresholdBasedOnConfigRegisterTest& instance() {
	static RxThresholdBasedOnConfigRegisterTest rc;
	return rc;
    }
    
    virtual void startTest(Sct_SctApi_T_SctApi& the_api, string& name, unsigned short& testVariable, unsigned long& nScans, vector<double>& testPoints) {	
	this->api = &the_api;
        // No masked channels
	name = "RxThresholdBasedOnConfigRegisterTest";
	testVariable = ST_ELAPSED;
	nScans = 1;
	// put in clock / 2 mode.
        
        // This next commented executable line (which used to put
        // the module into clock by two mode)
        // mode, and thus enables SendConfRegister mode) 
        // is the first line which (beyond the test names!) 
        // should be the source of difference between THIS
        // test and the usual RxThresholdTest:
	// APICALL(api, modifyABCDVar(&st, ST_FEEDTHROUGH, 0), "RxThresholdBasedOnConfigRegisterTest::startTest");
	testPoints.push_back(nTriggers);
    }
    
    virtual ScanRequest* getNextScan(unsigned long index) {
	ScanLibrary& sl = CalibrationControllerImpl::instance().getScanLibrary();
       const bool issueSoftReset=true;
	ScanRequest& request = sl.rawScan(nTriggers, issueSoftReset);

       // Number of bytes to read from the conf register: There should be about 210 ish bytes.  We will read a few more just so we have some elbow room.  Rather arbitrarily, then we chose to read 256 bits:
       const int bitsToRead = 256;
       request.setWidth(bitsToRead);
  	Sct_SctApi_T_Scan* scan = request.getScan();
	
       // This next two executable lines (which disables data
       // mode, and thus enables SendConfRegister mode) 
       // is the second line which (beyond the test names!) 
       // should be the source of difference between THIS
       // test and the usual RxThresholdTest:
       APICALL(scan, setOption(&st, Sct_SctApi_T_Scan_ScanOptions_enableDataMode, 0), "Failed to disable dataMode (failed to enable SendConfRegIDMode)")
          

	APICALL(scan, configure(&st, ST_RX_THRESHOLD, 0, 255, 5), "Failed to set Scan information")
       return &request;
    }
    
    virtual bool canFeedback() {
	return true;
    }
    
    virtual void endTest() {
      // put width back to whatever it was originally
      // APICALL(api, modifyABCDVar(&st, ST_FEEDTHROUGH, 1), "RxThresholdBasedOnConfigRegisterTest::endTest");
	api = 0;
    }

private:
    RxThresholdBasedOnConfigRegisterTest() {}
    Sct_SctApi_T_SctApi* api;    
    static const int nTriggers = 10;
};
}
}
#endif //SCTCALIBRATIONCONTROLLER_RXTHRESHOLDBASEDONCONFIGREGISTERTEST_H
