#ifndef SYNCHTRIGGERNOISETEST_H
#define SYNCHTRIGGERNOISETEST_H

#include <cmath>  // For fabs

#include "../ipc/Sct_CalibrationController.hh"
#include "../CalibrationControllerImpl.h"
#include "../ipc/ScanRequest.h"
#include "../SctApiAccessException.h"
#include "../SctApiCall.h"
#include "Sct/LogicErrors.h"

#include <CommonWithDsp/ABCD/ABCDscans.h>

using namespace Sct;
using namespace SctCalibrationController::Ipc;
namespace SctCalibrationController {
namespace Scripts {

class SynchTriggerNoiseTest  : public TestRequestImpl {
public:
    static SynchTriggerNoiseTest& instance() {
      static SynchTriggerNoiseTest stnt;
      return stnt;
    }

    virtual void startTest(Sct_SctApi_T_SctApi& api, string& name, unsigned short& testVariable, unsigned long& nScans, vector<double>& testPoints) {	
	this->api = &api;

        // Compression mode X1X
        APICALL(&api, modifyABCDVar(&st, ST_COMPRESSION, 1), "SynchTriggerNoiseTest::startTest")

        // Edge detect off
        APICALL(&api, modifyABCDVar(&st, ST_EDGE_DETECT, 0), "SynchTriggerNoiseTest::startTest")

        // No masked channels
        APICALL(&api, modifyABCDVar(&st, ST_SEND_MASK, 0), "SynchTriggerNoiseTest::startTest")

        // 1 fC threshold
        APICALL(&api, modifyABCDVar(&st, ST_QTHR, 1.0), "SynchTriggerNoiseTest::startTest")

	name = "SynchTriggerNoiseTest";
	testVariable = ST_ELAPSED;
	nScans = 1;
	testPoints.push_back(0.);
    }


    virtual ScanRequest* getNextScan(unsigned long index) {
        ScanLibrary& sl = CalibrationControllerImpl::instance().getScanLibrary();
        ScanRequestImpl& request = sl.defaultScan();

	// Create scan
        Sct_SctApi_T_Scan* s = 0;
        APIRETCALL(api, s, createScan(&st), "SynchTriggerNoiseTest::getNextScan")
        request.setScan(s);

	// Currently this is divided by 15 (nth)
	// At 100kHz 1e5 is 1 second
        APICALL(s, setNTrigs(&st, 100000), "Failed to set triggers")

        // This doesn't work with group distribution which is the default otherwise
	APICALL(s, setOption(&st, Sct_SctApi_T_Scan_ScanOptions_distSlave, 0), "Failed to set distSlave option")

	APICALL(s, setOption(&st, Sct_SctApi_T_Scan_ScanOptions_tim, 1), "Failed to set TIM option")
	APICALL(s, setOption(&st, Sct_SctApi_T_Scan_ScanOptions_nth, 15), "Failed to set nth option")

	// Adjust trigger
        Sct_SctApi_T_Trigger* t = 0;
        APIRETCALL(s, t, getTrigger1(&st), "Failed to get trigger1")

        APICALL(t, singleL1A(&st), "Failed to set trigger type")

	// Configure scan
        APICALL(s, configure(&st, ST_ELAPSED, 0, 5, 1), "Failed to set Scan information")

	return &request;
    }

    virtual bool canFeedback() {
	return false;
    }

    virtual void endTest() {
	api = 0;
    }

private:
    SynchTriggerNoiseTest() {}
    Sct_SctApi_T_SctApi* api;
};
    
}
}

#endif //SYNCHTRIGGERNOISETEST_H
