// Script to do a response curve
SI = GuiComponents.System.SystemInterface.getInstance();
api = SI.getSctApi();
cal = SI.getCalibrationController();
scanRequest = cal.getScanLibrary().defaultScan();
scan = scanRequest.getScan();

// Build the scan object
scan.setNTrigs(1000);
scan.setOption(Sct_SctApi.Scan.ScanOptions.loopCalLine, 1);

trigger = scan.getTrigger1();
trigger.calL1A(0x82);

// Pre-scan module configuration
sdesc = Sct.ScanDescriptors.getInstance();
api.modifyABCDVar(sdesc.valueFor("ST_COMPRESSION"), 1.0);
api.modifyABCDVar(sdesc.valueFor("ST_EDGE_DETECT"), 0.0);
api.modifyABCDVar(sdesc.valueFor("ST_SEND_MASK"), 0.0);

charges = {0.5, 0.75, 1.0, 1.25, 1.5, 2, 3, 4, 6, 8};

double lowLimit = 0;
double highLimit = 633;

for(int i=0; i<10; i++) {
    calChargefC = charges[i];

    api.modifyABCDVar(sdesc.valueFor("ST_QCAL"), calChargefC);

    offset = 0.0;  // This is passed into thresholdScan by default...

    lo = 40.0 * calChargefC + offset/2.;
    hi = 150.0 + 70.0 * calChargefC + offset;

    //Only set to lowLimit if we have some cal charge!
    if ((calChargefC > 0 && lo < lowLimit) || calChargefC < 0) lo = lowLimit;
    else if (lo < 0) lo = 0;
    if (hi > highLimit || calChargefC < 0) hi = highLimit;
    if (hi < lo) hi = lo + 100;
	
    scan.configure((short)sdesc.valueFor("ST_VTHR"), lo, hi, 5.0);

    cal.doScan(scanRequest);
}
