#include "Sct/Serializable.h"
#include <iostream>
#include <list>
#include <fstream>
#include <sstream>
#include "Sct/IS/IOManagerIS.h"
#include "Sct/File/IOManagerFile.h"
#include "Sct/Archive/IOManagerArchiveFile.h"
#include "Sct/IoExceptions.h"
#include "SctData/TestResult.h"
#include "SummaryWriter/SummaryManager.h"

std::string file;
std::list<std::string> names;

shared_ptr<Sct::Serializable> readObject(string name) {
    FILE* f = fopen(name.c_str(), "rb");
    if (f) {
	fclose(f);
	if (name.substr(name.size()-3) == ".gz"){
	  return Sct::Archive::IOManagerArchiveFile::instance().read(name);
	} else {
	  return Sct::File::IOManagerFile::instance().read(name);
	}
    } else {
	return Sct::IS::IOManagerIS::instance().read(name);
    }
}

void printHelp() {
  std::cerr << "Usage: SummaryExtractor [-file <filename>] <Names>" << std::endl;
  std::cerr << "Print the TestSummary information about a test" << std::endl;
  std::cerr << "Options: -file <filename> write to named file rather than to standard output" << std::endl;
}

void handleArgs(int argc, char** argv) {
  if (argc == 1) {
    printHelp(); 
    exit(0); 
  }
  for (int i=1; i<argc; ++i) {
    if (string(argv[i])=="-file"){
      if (i+1 >= argc) {
	std::cerr << "Invalid arguments for option -file" << std::endl;
	printHelp();
	exit(1);
      }
      file=argv[i+1];
      // std::cout << "File = " << file << std::endl;
      i++;
    }else{
      // std::cout << "Adding `" << argv[i] << std::endl;
      names.push_back(argv[i]);
    }
  }
}

void writeOut(std::ostream& os){
  for (std::list<string>::const_iterator it = names.begin();
       it!=names.end(); ++it){    
    try{
      boost::shared_ptr<Sct::Serializable> ob = readObject(*it);
      if (!ob.get()) throw Sct::IllegalStateError("readObject returned null", __FILE__, __LINE__);
      boost::shared_ptr<SctData::TestResult> tr = boost::dynamic_pointer_cast<SctData::TestResult>(ob);
      if (!tr.get()){
	std::ostringstream msg;
	msg << "Not a TestResult : " << (*it);
	throw Sct::IoException(msg.str(), __FILE__, __LINE__);
      }
      SctData::TestSummary::SummaryManager::instance().write(*tr, os);
    }catch(Sct::Throwable& e){
      std::cerr << "ERROR:" << e.what() << std::endl;
    }
  }
}

int main(int argc, char** argv){
  setExceptionHandlers(argv[0]);
  handleArgs(argc,argv);
  if (file==""){
    writeOut(std::cout);
  }else{
    std::ofstream f(file.c_str());
    if (!f.good()){
      std::cerr << "SummaryExtractor: Bad output file name " 
		<< file << std::endl;
      exit(1);
    }
    writeOut(f);
  }
  exit (0);
}
