#!/usr/bin/perl -w
use Fcntl;
use strict;
use CGI qw(param);


my $rootDirectory ="/datadisk/atlasdata2/SctData/Data/Barrel3/summaries/";
my $dataDirectory ="/datadisk/atlasdata2/SctData/Data/Barrel3/saved_data/";
my $lookupDirectory = "/userdisk/daquser/WebLookupData/";
my ($outputfile,$resultHeader);
my @chipnames = qw {M0 S1 S2 S3 S4 E5 M8 S9 S10 S11 S12 E13};
my $noDataFlag=".";

my @tests = ("StrobeDelay","NPtGain","NoiseOccupancy");
my @referencefiles = ("HybDelay_Reference_warm.txt","Hyb3PGain_Reference_warm.txt","HybNoise_Reference_warm.txt");

################################################################# Do not edit anything after this line

my %testmap=();
my %datamap=();
my %corrmap=();
my %corr_runno=();
my %refdata=();

for(my $i=0;$i<3;$i++) {

  $outputfile = $tests[$i] . "_summary2.txt";
  $resultHeader = $tests[$i] . "TestResult";
  %testmap=();
  %datamap=();
  %corrmap=();
  %corr_runno=();
  %refdata=();
  &collectLinks($i);
  &getReferenceData($i);
  &printList($i);
 }


### finished

sub bailout {

    my $error = "@_";

    print <<bailout_marker;
    <b> &gt Request Failed!</b>
    <br><br>
      Reason: $error
    <br><br>
    Sorry!
<br><br>
</body></html>
bailout_marker

die "$error";

}



sub collectLinks {

my ($index) = @_;


opendir(HDATA,$rootDirectory) || bailout("Cant open directory $rootDirectory: $!");

while(my $filename = readdir(HDATA)) {

   if($filename=~/$resultHeader\.(\d+)\.(\d+)\.(\d{14})\.summary.*/) {

      my $runno=$1;
      my $scanno = $2;
      my $sn = $3;
      
#      if($runno>1530) {next;}

      if($sn!~/20220(33|04|17|38)02\d{5}/) {next;}
      
      # Require 3PtGain only
      if($index==1) {
        my $nscans = getScanCount($filename);
	if($nscans>3) {next;}
	}

      if(!exists $testmap{$sn}) {
 	 $testmap{$sn}=$filename;
	 next;
	 }

      my $runscankey = $runno * 1e6 + $scanno;

      my $oldfile = $testmap{$sn};
      if($oldfile=~/$resultHeader\.(\d+)\.(\d+)\.(\d{14})\.summary.*/) {
         my $thisrun = $1;
	 my $thisscan=$2;
	 my $thissn=$3;
	 my $thiskey = $thisrun * 1e6 + $thisscan;
         if($runscankey > $thiskey) {$testmap{$sn}=$filename;}
	 }
      }
  }

closedir(HDATA);
}

sub getReferenceData() {
    my ($index) = @_;
    my $filename = $lookupDirectory . $referencefiles[$index];

    printf "filename=$filename\n";
    
    if(! -e $filename) {printf "**Reference file $filename does not exist\n"; return;}
    
    open(IN,$filename) || bailout("cant open $filename $!");
    flock(IN,1) || bailout("Cant flock $filename $!");

    while(<IN>) {
      if(/SN=(\d{14})TESTNO=(\d+)DATA=(.*)/) {
         if(exists $testmap{$1}) {$refdata{$1}=$3;}
	 }
      }
    close(IN) || bailout ("Cant close $filename $!");
    }


sub getCalCorrFactors {

 my ($sn,$runno) = @_;

#     print "getting cal corrs for run $runno for module $sn\n";
#     print "===================================================\n";
     
     my $configfile = $dataDirectory . "configArchive_${runno}_start.xml";
     my $xmlcontent = `gunzip -c $configfile`;
     
     my @xml = split(/[\n\r]/,$xmlcontent);
     my $chipno=0;
     my $corrline="";
     my $foundit=0;

     foreach my $line (@xml) {

       if($line=~/<sn>$sn<\/sn>.*/) {$foundit=1;}
       if($foundit==0) {next;}

       if($line=~/<c_factor>(.*)<\/c_factor>/) {
	  my $corrfactor=$1;
	  if($corrfactor=~/(^Z.*)/) {printf "**ERROR $sn $chipnames[$chipno] Cal factor $1\n";}
	  else {
	     if($corrline ne "") {$corrline .=";";}
	     $corrline .= "$chipnames[$chipno]:$corrfactor"; $chipno++;
	     }
	  }
       if($line=~/<\/module>.*/) {last;}
       }

      if($chipno!=12) {
             print "**ERROR - Run $runno module $sn chipno=$chipno\n";
	     return "";
	     }

#	print "RUN $runno $sn corrline=$corrline\n";
	     return "CFACTORS=$corrline";



}





sub printList {

my ($index) = @_;

my $modulecount=0;
my $outfile = "${lookupDirectory}$outputfile";

#open(OUTF,">$outfile") || bailout("Couldnt open $outfile: $!");
sysopen(OUTF,$outfile, O_RDWR) || bailout("Cant sysopen $outfile : $!");
flock(OUTF,2) || bailout("Cant flock $outfile $!");
seek(OUTF,0,0) || bailout("Cant seek $outfile: $!");
truncate(OUTF,0) || bailout("Cant truncate $outfile $!");

foreach my $sn (sort keys %testmap) {
   my $fname = $testmap{$sn};
   my $runno=0;
   my $scanno=0;
   if($fname=~/${resultHeader}\.(\d+)\.(\d+)\.(\d{14})\.summary.*/) {$runno=$1;$scanno=$2;}

   my $corrfactors = &getCalCorrFactors($sn,$runno);
#   print "$sn = $testmap{$sn}\n";
   my $file = $rootDirectory . $testmap{$sn};
   my $loop=0;
   my $defectCount=0;
   my $firstdefect=-1;
   my $T0=$noDataFlag;
   my $T1=$noDataFlag;
   my $VDET=$noDataFlag;
   my $IDET=$noDataFlag;
   my $VCC=$noDataFlag;
   my $ICC=$noDataFlag;
   my $VDD=$noDataFlag;
   my $IDD=$noDataFlag;
   my $nextLineIsTemperatures=0;
   my $nextLineIsHV=0;
   my $nextLineIsVCC=0;
   my $nextLineIsVDD=0;
   my $chipoffset=-1;
   my $line = "";
   open(IN,$file) || bailout("Cant open  file $file : $!");
    flock(IN,1) || bailout("Cant get LOCK_SH on $file: $!");
    my $nextline="";
    while(<IN>) {


      if(/^#T0\s*T1.*/) {$nextLineIsTemperatures=1; next;}
      if(/#VDET\s*IDET.*/) {$nextLineIsHV=1; next;}
      if(/#VCC\s*ICC.*/) {$nextLineIsVCC=1; next;}
      if(/#VDD\s*IDD.*/) {$nextLineIsVDD=1; next;}
      if($nextLineIsTemperatures==1) {
        if(/([\S]+)\s+([\S]+)/) {
	  $T0= ($1 eq ".") ? "." : sprintf("%.2f",$1);
	  $T1= ($2 eq ".") ? "." : sprintf("%.2f",$2);
	  }
	$nextLineIsTemperatures=0;
	}
      if($nextLineIsHV==1) {
        if(/([\S]+)\s+([\S]+)/) {
	  $VDET=$1;
	  $IDET=$2;
	  }
	$nextLineIsHV=0;
	}
      if($nextLineIsVDD==1) {
        if(/([\S]+)\s+([\S]+)/) {
	  $VDD=$1;
	  $IDD=$2;
	  }
	$nextLineIsVDD=0;
	}
      if($nextLineIsVCC==1) {
        if(/([\S]+)\s+([\S]+)/) {
	  $VCC=$1;
	  $ICC=$2;
	  }
	$nextLineIsVCC=0;
	}
	
    if($index==1) {                      #nptgain
      if(/^#LoopB.*/) {$loop++;}
      if($loop==0) {next;}
      if(/^#([MSE]\d{1,2})$/) {$nextline=$1; next;}
      if($nextline ne "") {
        my @params = split(/\s+/);
	my $nseoffset = (@params==9) ? 7 : 8;
	my $gainoffset = (@params==9) ? 2 : 3;
	my $nse = $params[$nseoffset];
	my $gain = $params[$gainoffset];

	if($line ne "") {$line .= ";";}
        $line .= "${nextline}:${nse}*$gain";
#	print "$sn $nextline = $nse\n";
	}
      $nextline="";
      }
    elsif($index==2) {                      #noise

      if(/^#([MSE]\d{1,2})$/) {$nextline=$1; next;}
      if($nextline ne "") {
        my @params = split(/\s+/);
	my $occoffset = (@params == 4) ? 1 : 2;
	my $nseoffset = (@params == 4) ? 3 : 4;
	my $occ = $params[$occoffset];
	my $nse = $params[$nseoffset];
	if($line ne "") {$line .= ";";}
        $line .= "${nextline}:${nse}*$occ";
#	print "$sn $nextline = $nse\n";
	}
      $nextline="";
      }
    elsif($index==0) {                    # strobedelay
      if(/^#M0\s+S1\s+S2\s+S3\s+S4\s+E5.*/) {$chipoffset=0; next;}
      if(/^#M8\s+S9\s+S10\s+S11\s+S12\s+E13.*/) {$chipoffset=6; next;}
      if($chipoffset != -1) {
         my @chipdata = split(/\s+/);
	 my $miny=$chipoffset;
	 my $maxy=6+$chipoffset;
	 for(my $y=$miny;$y<$maxy;$y++) {
	    if($line ne "") {$line .= ";";}
	    $line .=$chipnames[$y] . ":$chipdata[$y-$chipoffset]";
	    }

         }
      $chipoffset=-1;
      }

      if(/#(\d+)\s+defects\s+found\s+affecting\s+(\d+)\s+strips.*/) {$defectCount=$2;}



      }
  printf "$tests[$index] Run $runno Scan $scanno $sn\n";

  my $DCS="${T0}*${T1}*${VDET}*${IDET}*${VCC}*${ICC}*${VDD}*${IDD}";
  
  my $REFDATA = (exists $refdata{$sn}) ? "REF=".$refdata{$sn} : "";

  print OUTF "RUN=${runno}SCAN=${scanno}SN=${sn}DATA=${line}DCS=${DCS}DEFECTS=${defectCount}${corrfactors}${REFDATA}\n";
  $modulecount++;


  close(IN) || bailout("Cant close file $file : $!");
  


   }
    printf "Saved $tests[$index] data for $modulecount modules\n";
    close(OUTF) || bailout("Cant close $outfile : $!");
}

sub getScanCount() {
 my $count=0;
 my $fname = shift;
 my $fullname = $rootDirectory . $fname;
 open(IN,$fullname) || bailout("Cant open $fullname");
 flock(IN,1) || bailout("Cant flock $fullname");
 
 my $nextLineIsCount=0;

 while(<IN>) {
   if(/^[\s]?#N_POINTS/) {$nextLineIsCount=1; next;}
   if($nextLineIsCount) {
     if(/[\s]?(\d+)/) {
       $count=$1;
       last;
       }
     }
   }
 close(IN) || bailout("Cant close $fullname");
 return $count;
 }






