#!/usr/bin/perl -w
use Fcntl;
use strict;
use CGI qw(param);

# Generate summary web page of sctdaq test results
# 25/03/2003 Dave R First version.
# 26/03/2003 Dave R List also non-production modules. 
#                   Improve code layout.
#                   Explicitly search for Module_Reception and Module_Longterm.
#

# Please modify the next line to your top directory
# ie so that it contains the sub-directories of the form 202203302nnnnn
# in my case I have directories like
#  /pchome/silicon/www/20220330200018/Module_Reception/index.html
# so my top directory is /pchome/silicon/www/
#
my $rootDirectory ="/userdisk/daquser/public_html/atlasdata/Reception_Test/SCTDAQ_results/";

my $daqLookupTable = "/userdisk/daquser/WebLookupData/moduleMounts";

# This is the URL corresponding to your top directory
my $url = "http://www-pnp.physics.ox.ac.uk/~daquser/atlasdata/Reception_Test/SCTDAQ_results/";

# uncomment your institute name
my $instituteName = "Oxford";

my $rootDir = "/userdisk/daquser/public_html/";
my $baseURL = "http://www-pnp.physics.ox.ac.uk/~daquser/";

my $showMapScript = "http://www-pnp.physics.ox.ac.uk/~daquser/cgi-bin/showModuleMap.cgi";



################################################################# Do not edit anything after this line
my %uklinks = ();
my %uslinks = ();
my %scandlinks = ();
my %japanlinks = ();
my %otherlinks = ();

my %daqInfo = ();
my %daqMapping=();
my $daqInfoExists=0;

my %moduleOrigin=();
my $originInfoExists=0;
my $moduleMountMapIsValid=0;

my $UKTitle = "UK-B Module Results";
my $USATitle = "USA Module Results";
my $SCANDTitle = "Scand Module Results";
my $JAPANTitle = "Japan Module Results";

my %statistics=();
$statistics{$UKTitle}{"GOOD"}=0;
$statistics{$USATitle}{"GOOD"}=0;
$statistics{$SCANDTitle}{"GOOD"}=0;
$statistics{$JAPANTitle}{"GOOD"}=0;
$statistics{$UKTitle}{"HOLD"}=0;
$statistics{$USATitle}{"HOLD"}=0;
$statistics{$SCANDTitle}{"HOLD"}=0;
$statistics{$JAPANTitle}{"HOLD"}=0;

my @assembledCount = (0,0,0,0);
my @barrelCount = (0,0,0,0);

my @barrelRows = (32,40,48,56);


my %linkInfo=();

my $LTTkey="LTT_TEST";
my $RECkey="RECEPTION_TEST";
my $STATUSkey ="STATUS";

my ($minute, $hour, $day, $month, $year) = (localtime)[1,2,3,4,5];
$year += 1900;
$month += 1;
$minute = "0" . $minute if ($minute<10);
$hour = "0" . $hour if ($hour<10);
$month = "0" . $month if ($month<10);
$day = "0" . $day if ($day<10);
my $shortyear = substr($year,2,3);
my $theDate = "$day/$month/$shortyear";
my $thelongDate = "$day/$month/$year";
my $datestamp = "$hour:$minute on $day/$month/$year";

# print "Content-type: text/html\n\n";

&doJavaDownloads();  ## download from SCT database lookup of module cluster origin

&readDaqTable();   ## read module assignment to barrels


&printHeader();
&collectLinks();

if((keys %uklinks)>0) {&getStatistics(0);}  # UK
if((keys %uslinks)>0) {&getStatistics(1);}  # USA
if((keys %scandlinks)>0) {&getStatistics(2);}  # Scand
if((keys %japanlinks)>0) {&getStatistics(3);}  # Japan

&printSummary();

if((keys %uklinks)>0) {&printTable(0);}  # UK
if((keys %uslinks)>0) {&printTable(1);}  # USA
if((keys %scandlinks)>0) {&printTable(2);}  # Scand
if((keys %japanlinks)>0) {&printTable(3);}  # Japan
if((keys %otherlinks)>0) {&printTable(4);}  # other links


print qq!
<br>
<br><br>
</body>
</html>
!;



### finished

sub bailout {

    my $error = "@_";

    print <<bailout_marker;
    <b> &gt Request Failed!</b>
    <br><br>
      Reason: $error
    <br><br>
    Sorry! 
<br><br>
</body></html>
bailout_marker
	
die "$error";

}

sub readDaqTable() {

    for(my $i=0;$i<4;$i++) {
      my $lookup = "$daqLookupTable$i".".txt";
      if(!(-e "$lookup")) {next;}
 
      open(IN,$lookup) || bailout("Cant open $lookup: $!");
      flock(IN,1) || bailout("Cant get LOCK_SH on $lookup: $!");

      while(<IN>) {
        if(/(\d{14})=Barrel\s*(\d+),\s*Row\s*(\d+),\s*Position\s*([-\d]+)/) {
            $daqInfo{$1}="Barrel $2, Row $3, Position $4";
            $barrelCount[$2-3]++;
            my $position = $4;
            $position+=6;
            if($position>=7) {$position--;}
            my $newpos = ($position + ($3 * 12))<<4;
            my $index = $2 + $newpos;
            $daqMapping{$index} = $1;
            $daqInfoExists++;
            }
          }
      close(IN) || bailout("Cant close $lookup: $!");
      }
    }

sub addLinks() {
   my $serialno=$_[0];
   my $thisURL=$_[1];
   my $keyType=$_[2];
   my $thisModuleType=$_[3];

   my $UKRegex = "RAL|Cambridge|Birmingham|London,QMW";
   my $JapanRegex = "KEK";
   my $ScandRegex="Uppsala|Bergen";
   my $USARegex="Berkeley LBL";

   my $originFlag=4;
   if($originInfoExists==1) {
        my $testinstitute = (exists $moduleOrigin{$serialno}) ? $moduleOrigin{$serialno} : "unknown";
        if($testinstitute=~/$UKRegex/) {$originFlag=0;}
        elsif($testinstitute=~/$USARegex/) {$originFlag=1;}
        elsif($testinstitute=~/$ScandRegex/) {$originFlag=2;}
        elsif($testinstitute=~/$JapanRegex/) {$originFlag=3;}
      }
   else {
       if($thisModuleType=~/33/) {$originFlag=0;}
       elsif($thisModuleType=~/04/) {$originFlag=1;}
       elsif($thisModuleType=~/38/) {$originFlag=2;}
       elsif($thisModuleType=~/17/) {$originFlag=3;}
       }

   if($originFlag==0) {
     if(exists $uklinks{$serialno}{$keyType}) {$uklinks{$serialno}{$keyType} .= "<br><a href=\"$thisURL$linkInfo{INDEX}\">$linkInfo{DATE}</a>";}
     else {$uklinks{$serialno}{$keyType} = "<a href=\"$thisURL$linkInfo{INDEX}\">$linkInfo{DATE}</a>"; }  
     }
   elsif($originFlag==1) {
     if(exists $uslinks{$serialno}{$keyType}) {$uslinks{$serialno}{$keyType} .= "<br><a href=\"$thisURL$linkInfo{INDEX}\">$linkInfo{DATE}</a>";}
     else {$uslinks{$serialno}{$keyType} = "<a href=\"$thisURL$linkInfo{INDEX}\">$linkInfo{DATE}</a>"; }
     }
   elsif($originFlag==2) {
     if(exists $scandlinks{$serialno}{$keyType}) {$scandlinks{$serialno}{$keyType} .= "<br><a href=\"$thisURL$linkInfo{INDEX}\">$linkInfo{DATE}</a>";}
     else {$scandlinks{$serialno}{$keyType} = "<a href=\"$thisURL$linkInfo{INDEX}\">$linkInfo{DATE}</a>"; }
     }
   elsif($originFlag==3) {
     if(exists $japanlinks{$serialno}{$keyType}) {$japanlinks{$serialno}{$keyType} .= "<br><a href=\"$thisURL$linkInfo{INDEX}\">$linkInfo{DATE}</a>";}
     else {$japanlinks{$serialno}{$keyType} = "<a href=\"$thisURL$linkInfo{INDEX}\">$linkInfo{DATE}</a>"; }
     }
   else {
     if(exists $otherlinks{$serialno}{$keyType}) {$otherlinks{$serialno}{$keyType} .= "<br><a href=\"$thisURL$linkInfo{INDEX}\">$linkInfo{DATE}</a>";}
     else {$otherlinks{$serialno}{$keyType} = "<a href=\"$thisURL$linkInfo{INDEX}\">$linkInfo{DATE}</a>"; }  
     }
}

sub printTable {

my $type=$_[0];


my $title="";
my $tag = "";
my %links = %uklinks;

if($type==0) {
   $title = $UKTitle;
   $tag="<a name=UKModules></a>"
   }
elsif($type==1) {
   $title = $USATitle; 
   %links=%uslinks;
   $tag="<a name=USModules></a>";
   }
elsif($type==2) {
   $title = $SCANDTitle; 
   %links=%scandlinks;
   $tag="<a name=SCANDModules></a>";
   }
elsif($type==3) {
   $title = $JAPANTitle;
   %links=%japanlinks;
   $tag="<a name=JAPANModules></a>";
   }
else {
   $title = "Modules with missing database records:";
   %links=%otherlinks;
   $tag="<a name=OTHERModules></a>";
   }

print qq!
$tag
<br>
<h3>$title</h3>
<table cellspacing="0" cellpadding="0" border="0">
<col width=130 align="left">
<col width=100 align="left">
<col width=80 align="left">
<col width=150 align="left">

<tr><td><spacer type=BLOCK WIDTH=130 HEIGHT=10></td><td><spacer type=BLOCK WIDTH=100 HEIGHT=10></td><td><spacer type=BLOCK WIDTH=80 HEIGHT=10></td><td><spacer type=BLOCK WIDTH=150 HEIGHT=10></td></tr>
<tr><td><b>Serial Number</b></td><td nowrap><b>Test Results</b></td><td nowrap><b>Status</b></td><td nowrap><b>Barrel Assignment</b></td></tr>
!;

my $detcount=0;
my $goodcount=0;
my $badcount=0;

  foreach my $sn (sort keys %links) {
     my $snlink=$sn;
     if($type==0) {$snlink="<a href=\"http://hepunx.rl.ac.uk/atlassct/cgibin/ModuleSummary.pl?serial=$sn\" target=\"ModuleSummary\">$sn</a>";}
     elsif($type==2) {$snlink="<a href=\"http://www4.tsl.uu.se/~Atlas/Electrical/$sn/$sn.html\" target=\"ModuleSummary\">$sn</a>";}
     elsif($type==1) {$snlink="<a href=\"http://www-atlas.lbl.gov/cgi-bin/folders.pl?DUT=$sn\" target=\"ModuleSummary\">$sn</a>";}
     my $receptionLink = (exists $links{$sn}{$RECkey}) ? $links{$sn}{$RECkey} : "None";

     my $subdir = "$rootDirectory$sn"."/";
     my $statusLink="";

     if($receptionLink ne "None") {
       my $holdfile = "$subdir"."/hold.txt";
       if(-e $holdfile) {$statusLink = "<a href=\"$url$sn"."/hold.txt\" class=\"red\">HOLD</a>";}
       else {$statusLink = "<font color=green>Good</font>";}
       }
     else {$statusLink="Pending";}
     my $background = ($detcount%2==0) ? "bgcolor=\"#f6f6f6\"" : "bgcolor=\"#eeeeff\"";
     my $defaultTxt = ($daqInfoExists>0) ? "None" : "Data not available";
     my $usedFlag = $defaultTxt;
     if(exists $daqInfo{$sn}) {
       if($daqInfo{$sn}=~/Barrel\s+(\d+).*/) {
          my $index=$1-2;
	  $usedFlag = "<a href=\"#\" onClick=\"window.open('$showMapScript\?type=moduleMounts\&index=$index','BarrelMapping','menubar=1,scrollbars=1'); return false;\">$daqInfo{$sn}</a>";
#          $usedFlag = "<a href=\"#\" onClick=\"window.open('$showMapScript\?type=moduleMounts\&index=$index','Barrel Map'); return false;\">$daqInfo{$sn}</a>";
	  }
#        if($daqInfo{$sn}=~/Barrel\s+(\d+).*/) {$usedFlag = "<a href=\"${baseURL}barrel$1.html#$sn\" target=\"barrelMap\">$daqInfo{$sn}</a>";}
        }
     print "<tr $background><td align=left valign=top>$snlink</td><td align=left valign=top>$receptionLink</td><td align=left valign=top>$statusLink</td><td align=left valign=top>$usedFlag</td></tr>\n";
     $detcount++;
     }

print "<tr><td colspan=3><br></td></tr>";
if($type<4) {print "<tr><td colspan=3 align=left>$title : $statistics{$title}{GOOD} are ok and $statistics{$title}{HOLD} are on HOLD.</td></tr>";}
print "<tr><td colspan=4><br></td></tr>";
print "<tr><td colspan=4><hr></td></tr>";
print "</table>";

}

sub getStatistics {

my $type=$_[0];


my $title="";

my %links = %uklinks;

if($type==0) {
   $title = $UKTitle;
   }
elsif($type==1) {
   $title = $USATitle; 
   %links=%uslinks;
   }
elsif($type==2) {
   $title = $SCANDTitle; 
   %links=%scandlinks;
   }
elsif($type==3) {
   $title = $JAPANTitle;
   %links=%japanlinks;
   }
else {
   $title = "Non-Production Module Results";
   %links=%otherlinks;
   }


my $detcount=0;
my $goodcount=0;
my $badcount=0;

  foreach my $sn (sort keys %links) {

     if($daqInfoExists>0 && exists $daqInfo{$sn}) {$assembledCount[$type]++;}     

     my $subdir = "$rootDirectory$sn"."/";
     my $receptionLink = (exists $links{$sn}{$RECkey}) ? $links{$sn}{$RECkey} : "None";

     if($receptionLink ne "None") {
       my $flag="HOLD";
       my $holdfile = "$subdir"."/hold.txt";
       if(-e $holdfile) {$flag="HOLD";}
       else {$flag="GOOD";}
       if(exists $statistics{$title}{$flag}) {$statistics{$title}{$flag}++;}
       else {$statistics{$title}{$flag}=1;}
       }
     $detcount++;
     }


}


sub printHeader {
print qq!
<HTML>
<HEAD>
   <TITLE>ATLAS SCT Barrel Module Status at $instituteName</TITLE>
<link rel=stylesheet HREF="http://www.hep.phy.cam.ac.uk/~silicon/styles/davestyles1.css" type="text/css">
</HEAD>
<BODY>
<h3>ATLAS SCT Barrel Module Status at $instituteName</h3>
<p>

<b>How to use this page</b><br>
This page provides links to the results of all module reception tests performed at $instituteName, as well as links to other test sites.<br>
&gt To view test results for a module, click on the <i>date of the test</i>.<br>
&gt To view the clusters own summary information for a module (if available), click on the <i>serial number</i>.<br>
&gt If a module is flagged as <font color=red>HOLD</font>, click on the Status link for further information.<br>
&gt If a module is mounted on a barrel, click on the Barrel Assignment link to view its location.<br>
!;

}

sub printSummary {
my $noUK = (keys %uklinks);
my $noUS = (keys %uslinks);
my $noScand = (keys %scandlinks);
my $noJapan = (keys %japanlinks);

my $total = $noUK + $noUS + $noScand + $noJapan;
my $totalgood = $statistics{$UKTitle}{GOOD} + $statistics{$USATitle}{GOOD} + $statistics{$SCANDTitle}{GOOD} + $statistics{$JAPANTitle}{GOOD};
my $totalhold = $statistics{$UKTitle}{HOLD} + $statistics{$USATitle}{HOLD} + $statistics{$SCANDTitle}{HOLD} + $statistics{$JAPANTitle}{HOLD};

my $uklink = ($noUK>0) ? "<a href=\"#UKModules\">List</a>" : "Not available";
my $uslink = ($noUS>0) ? "<a href=\"#USModules\">List</a>" : "Not available";
my $scandlink = ($noScand>0) ? "<a href=\"#SCANDModules\">List</a>" : "Not available";
my $japanlink = ($noJapan>0) ? "<a href=\"#JAPANModules\">List</a>" : "Not available";

my $ukcount = ($daqInfoExists>0) ? $assembledCount[0] : "Data not available";
my $uscount = ($daqInfoExists>0) ? $assembledCount[1] : "Data not available";
my $scandcount = ($daqInfoExists>0) ? $assembledCount[2] : "Data not available";
my $japancount = ($daqInfoExists>0) ? $assembledCount[3] : "Data not available";

my $totalassembled = "Data not available";
if($daqInfoExists>0) {
   $totalassembled = $ukcount + $uscount + $scandcount + $japancount;
   }

my $otext = ($originInfoExists==0) ? "<b>SCT database info not available - modules grouped by serial number only.</b>" : "Cluster origin of modules confirmed from SCT Database.";


print qq!
<br>
<table cellspacing="0" cellpadding="0" border="0">
<col width=90 align="left">
<col width=80 align="left">
<col width=80 align="left">
<col width=80 align="left">
<col width=80 align="left">
<col width=100 align="left">

<tr><td><b>Cluster</b></td><td><b>Data</b></td><td><b>#Tested</b></td><td><b>#Good</b></td><td><b>#on HOLD</b></td><td><b>#Mounted</b></td></tr>
<tr><td colspan=6><hr></td></tr>
<tr><td><a href="http://hepunx.rl.ac.uk/atlassct/ModulePublic.html" target="clusters">UK</a></td><td>$uklink</td><td>$noUK</td><td>$statistics{$UKTitle}{GOOD}</td><td>$statistics{$UKTitle}{HOLD}</td><td>$ukcount</td></tr>
<tr><td><a href="http://www-atlas.lbl.gov/strips/modules/production.html" target="clusters">US</a></td><td>$uslink</td><td>$noUS</td><td>$statistics{$USATitle}{GOOD}</td><td>$statistics{$USATitle}{HOLD}</td><td>$uscount</td></tr>
<tr><td><a href="http://www4.tsl.uu.se/~Atlas/Electrical/ProductionMain.html" target="clusters">Scand</a></td><td>$scandlink</td><td>$noScand</td><td>$statistics{$SCANDTitle}{GOOD}</td><td>$statistics{$SCANDTitle}{HOLD}</td><td>$scandcount</td></tr>
<tr><td>Japan</td><td>$japanlink</td><td>$noJapan</td><td>$statistics{$JAPANTitle}{GOOD}</td><td>$statistics{$JAPANTitle}{HOLD}</td><td>$japancount</td></tr>
<tr><td colspan=6><br></td></tr>
<tr><td>Totals:</td><td></td><td>$total</td><td>$totalgood</td><td>$totalhold</td><td>$totalassembled</td></tr>
<tr><td colspan=6><hr></td></tr>
</table>
<br>
$otext
<br>
This page was last updated at $datestamp
<br>
<br>
<b>Barrel Module Position Maps:</b>&nbsp;&nbsp;&nbsp;
!;
if($moduleMountMapIsValid==0) {
  print "Not available (SCTDB download failed)";
  }
else {
#my $sname = $showMapScript . "\?type=moduleMounts\&index=1";
#print "<a href=\"#\" onClick=\"window.open('$sname','BarrelMapping'); return false;\">Barrel 3</a>";
#print "&nbsp;&nbsp;-&nbsp;&nbsp";
#$sname = $showMapScript . "\?type=moduleMounts\&index=2";
#print "<a href=\"#\" onClick=\"window.open('$sname','BarrelMapping'); return false;\">Barrel 4</a>";
#print "&nbsp;&nbsp;-&nbsp;&nbsp";
#$sname = $showMapScript . "\?type=moduleMounts\&index=3";
#print "<a href=\"#\" onClick=\"window.open('$sname','BarrelMapping'); return false;\">Barrel 5</a>";
#print "&nbsp;&nbsp;-&nbsp;&nbsp";
#$sname = $showMapScript . "\?type=moduleMounts\&index=4";
#print "<a href=\"#\" onClick=\"window.open('$sname','BarrelMapping'); return false;\">Barrel 6</a><br><br>";

  for(my $y=1;$y<=4;$y++) {
    my $sname = $showMapScript . "\?type=moduleMounts\&index=$y";
    my $bindex=$y+2;
    print "<a href=\"#\" onClick=\"window.open('$sname','BarrelMapping','menubar=1,scrollbars=1'); return false;\">Barrel $bindex</a>";

     if($y<4) {print "&nbsp;&nbsp;-&nbsp;&nbsp";}
     }
   }
   print "<br><br>";
}


sub collectLinks {


opendir(HDATA,$rootDirectory) || bailout("Cant open directory $rootDirectory: $!");

while(my $serialno = readdir(HDATA)) {

      if($serialno =~/20220(\d{2})0(1|2)\d{5}/ && -d "$rootDirectory$serialno") {
          my $thisDirectory = "$rootDirectory$serialno";
          my $thisModuleType=$1;
 


          opendir(RESULTS,$thisDirectory) || bailout("Cant open $thisDirectory : $!");

          while(my $thisname = readdir(RESULTS)) {
             if($thisname!~/Module_Oxford\d*/) {next;}   # better check explicitly...
             my $thisURL = "$url$serialno"."/".$thisname;

 

             if(-d "$rootDirectory$serialno"."/".$thisname) {
   


                   my $fulldirectory = "$rootDirectory$serialno"."/".$thisname;
                   opendir(RESULTS2,$fulldirectory) || bailout("Cant open $fulldirectory: $!");
                   %linkInfo=();
                   while(my $rfiles = readdir(RESULTS2)) {
                          if($rfiles=~/\d{14}_(\d{4})(\d{2})(\d{2})_\d+/) {$linkInfo{DATE}="$3/$2/$1";}
                          if($rfiles=~/index.html/) {$linkInfo{INDEX}="/index.html";}
                          }
                   closedir(RESULTS2);
                   if(exists $linkInfo{DATE} && exists $linkInfo{INDEX}) {
                          my $keyType = $RECkey;
                          &addLinks($serialno,$thisURL,$keyType,$thisModuleType);
                          }                   
                    }

             }
          closedir(RESULTS)
         }
}

closedir(HDATA);
}

sub doJavaDownloads() {
my $oldpath = $ENV{"PATH"};
my $cp = "/userdisk/daquser/javaclasses/DisplayGUI.jar:/userdisk/daquser/javaclasses/classes111.zip";
#$ENV{"PATH"}=$oldpath;

my $list = `/usr/java/j2sdk1.4.1_02/bin/java -cp $cp webscripts/getTable`;

my @locs = split(/[\n\r]/,$list);
my $count=0;
foreach my $line (@locs) {
  my ($sn,$loc) = split(/:/,$line);
  $moduleOrigin{$sn}=$loc;
  $count++;
  }

if($count>1) {$originInfoExists=1;}

my $download = `/usr/java/j2sdk1.4.1_02/bin/java -cp $cp webscripts/getAssemblyMaps /userdisk/daquser/WebLookupData`;
$moduleMountMapIsValid = ($download=~/.*Completed.*/) ? 1 : 0;


#print $list;
}





