#!/usr/bin/perl -w

#
# Barrel Data display script 
# Dave Robinson
#

use Fcntl;
use strict;
use CGI qw(param);

#use CGI::Carp qw(fatalsToBrowser);

my $daqLookupTable = "/userdisk/daquser/WebLookupData/";

my $maintenancefile = "/userdisk/daquser/WebLookupData/maintenancefile.txt";

# This is the URL corresponding to your top directory
my $url = "http://www-pnp.physics.ox.ac.uk/~daquser/atlasdata/Reception_Test/SCTDAQ_results/";

my $logScript = "http://www-pnp.physics.ox.ac.uk/~daquser/cgi-bin/sctroddaqsummary.cgi";

# uncomment your institute name
my $instituteName = "Oxford";

my $rootDir = "/userdisk/daquser/public_html/";
my $baseURL = "http://www-pnp.physics.ox.ac.uk/~daquser/";
my $summaryDir = "/datadisk/atlasdata2/SctData/Data/Barrel3/summaries/";

my @sctViews =qw{"Barrel 3" "Barrel 4" "Barrel 5" "Barrel 6"};
my @chipNames = ("Module Average","Chip Maximum","Chip Minimum","Mean Link0","Mean Link1","Chip M0","Chip S1","Chip S2","Chip S3","Chip S4","Chip E5","Chip M8","Chip S9","Chip S10","Chip S11","Chip S12","Chip E13");

my @nptgain_param_names = ("ENC Noise","Gain (mV/fC)","ENCnoise (SctRodDaq-SCTDAQ)","Gain (SctRodDaq-SCTDAQ)","Defective Channels","DCS Data - T0","DCS Data - T1","DCS Data - VDet","DCS Data - IDet(nA)","DCS Data - Vcc","DCS Data - Icc","DCS Data - Vdd","DCS Data - Idd","Hybrid TDiff");
my @occ_param_names = ("ENC Noise","Occupancy","ENCnoise (SctRodDaq-SCTDAQ)","Occupancy (SctRodDaq-SCTDAQ)","Defective Channels","DCS Data - T0","DCS Data - T1","DCS Data - VDet","DCS Data - IDet(nA)","DCS Data - Vcc","DCS Data - Icc","DCS Data - Vdd","DCS Data - Idd","Hybrid TDiff");
my @delay_param_names = ("Delay","Delay (SctRodDaq-SCTDAQ)","Defective Channels","DCS Data - T0","DCS Data - T1","DCS Data - VDet","DCS Data - IDet(nA)","DCS Data - Vcc","DCS Data - Icc","DCS Data - Vdd","DCS Data - Idd","Hybrid TDiff");
my @cellfills = ("Data Value","Serial No","cFactor","None");

my $scriptname = $ENV{'SCRIPT_NAME'};  # name of this script for links etc

my $noDataString = "No Data";

my @barrelCount = (0,0,0,0);
my @barrelRows = (32,40,48,56);
my @testIndexNames = qw{NPtGain NoiseOccupancy StrobeDelay};
my @parameterIndexNames = qw{enc occ};

my %colorHashC0 = (
   "ENC Noise" => [1200, 0, 0, 255 ],    # label then rgb in 0-255
   "Gain (mV/fC)" => [30,0,0,255],
   "Defective Channels" => [0,0,255,0],
   "DCS Data - T0" => [0,0,0,255],
   "DCS Data - T1" => [0,0,0,255],
   "DCS Data - VDet" => [0,0,0,255],
   "DCS Data - IDet(nA)" => [0,0,0,255],
   "Hybrid TDiff" => [0,0,255,0],
   "DCS Data - Vdd" => [3.5,0,0,255],
   "DCS Data - Idd" => [450,0,0,255],
   "DCS Data - Vcc" => [3.0,0,0,255],
   "DCS Data - Icc" => [800,0,0,255],
   "Delay" => [0,0,0,255],
   "Occupancy" => [0.0000001,0,0,255],
   "ENCnoise (SctRodDaq-SCTDAQ)" => [-150,0,0,255],
   "Gain (SctRodDaq-SCTDAQ)" => [-10,0,0,255],
   "Occupancy (SctRodDaq-SCTDAQ)" => [-0.00005,0,0,255],
   "Delay (SctRodDaq-SCTDAQ)" => [-5,0,0,255]
   );
   
my %colorHashC1 = (
   "ENC Noise" => [1400, 0, 255, 0 ],    # label then rgb in 0-255
   "Gain (mV/fC)" => [40,0,255,0],
   "Defective Channels" => [5,255,255,0],
   "DCS Data - T0" => [20,0,255,0],
   "DCS Data - T1" => [20,0,255,0],
   "Hybrid TDiff" => [1,255,255,0],
   "DCS Data - VDet" => [150,0,255,0],
   "DCS Data - IDet(nA)" => [400,0,255,0],
   "DCS Data - Vdd" => [3.8,0,255,0],
   "DCS Data - Idd" => [500,0,255,0],
   "DCS Data - Vcc" => [3.4,0,255,0],
   "DCS Data - Icc" => [950,0,255,0],
   "Delay" => [10,0,255,0],
   "Occupancy" => [0.000001,0,255,0],
   "ENCnoise (SctRodDaq-SCTDAQ)" => [-75,0,255,0],
   "Gain (SctRodDaq-SCTDAQ)" => [-5,0,255,0],
   "Occupancy (SctRodDaq-SCTDAQ)" => [-0.00002,0,255,0],
   "Delay (SctRodDaq-SCTDAQ)" => [-2,0,255,0]
   );
my %colorHashC2 = (
   "ENC Noise" => [1600, 0, 255, 0 ],    # label then rgb in 0-255
   "Gain (mV/fC)" => [60,0,255,0],
   "Defective Channels" => [10,255,255,0],
   "DCS Data - T0" => [30,0,255,0],
   "DCS Data - T1" => [30,0,255,0],
   "Hybrid TDiff" => [3,255,255,0],
   "DCS Data - VDet" => [350,0,255,0],
   "DCS Data - IDet(nA)" => [600,0,255,0],
   "DCS Data - Vdd" => [4.2,0,255,0],
   "DCS Data - Idd" => [550,0,255,0],
   "DCS Data - Vcc" => [3.6,0,255,0],
   "DCS Data - Icc" => [1050,0,255,0],
   "Delay" => [20,0,255,0],
   "Occupancy" => [0.00001,0,255,0],
   "ENCnoise (SctRodDaq-SCTDAQ)" => [75,0,255,0],
   "Gain (SctRodDaq-SCTDAQ)" => [5,0,255,0],
   "Occupancy (SctRodDaq-SCTDAQ)" => [0.00002,0,255,0],
   "Delay (SctRodDaq-SCTDAQ)" => [2,0,255,0]
   );

my %colorHashC3 = (
   "ENC Noise" => [1800, 255, 0, 0 ],    # label then rgb in 0-255
   "Gain (mV/fC)" => [70,255,0,0],
   "Defective Channels" => [15,255,0,0],
   "DCS Data - T0" => [40,255,0,0],
   "DCS Data - T1" => [40,255,0,0],
   "Hybrid TDiff" => [4,255,0,0],
   "DCS Data - VDet" => [500,255,0,0],
   "DCS Data - IDet(nA)" => [1000,255,0,0],
   "DCS Data - Vdd" => [4.5,255,0,0],
   "DCS Data - Idd" => [600,255,0,0],
   "DCS Data - Vcc" => [4.0,255,0,0],
   "DCS Data - Icc" => [1200,255,0,0],
   "Delay" => [30,255,0,0],
   "Occupancy" => [0.0001,255,0,0],
   "ENCnoise (SctRodDaq-SCTDAQ)" => [150,255,0,0],
   "Gain (SctRodDaq-SCTDAQ)" => [10,255,0,0],
   "Occupancy (SctRodDaq-SCTDAQ)" => [0.00005,255,0,0],
   "Delay (SctRodDaq-SCTDAQ)" => [5,255,0,0]
   );




################################################################# Do not edit anything after this line

print "Content-type: text/html\n\n";

if(-e $maintenancefile) {
 print qq!
 <html>
 <head>
 <title>Barrel Display not available</title>
 </head>
 <body>
 <h3>The Barrel Display is not available</h3>
 Sorry, the barrel data display is not available, due to maintenance.
 <br><br>
 Please try again in a few moments.
 <br><br>
 If this message persists, please <a href="mailto:robinson\@hep.phy.cam.ac.uk>complain</a>.
 </body>
 </html>
 !;
 die;
 }

my %daqInfo = ();
my %daqMapping=();
my %dataMap=();
my %cMap=();
my $daqInfoExists=0;

my %datalinks=();
my $originInfoExists=0;
my $moduleMountMapIsValid=0;

my $moduleCount=0;


# index = barrel number
# testIndex = index of test name
# parameterIndex = index of displayed parameter
# chip = index of chip option
my $index = param("index") ? param("index") : 1;
my $testIndex = param("testIndex") ? param("testIndex") : 0;
my $parameterIndex = param("parameterIndex") ? param("parameterIndex") : 0;
my $chipIndex = param("chipIndex") ? param("chipIndex") : 0;
my $cellfillIndex = param("cellfillIndex") ? param("cellfillIndex") : 0;

my @paramlist = @nptgain_param_names;
if($testIndex==1) {@paramlist = @occ_param_names;}
elsif($testIndex==2) {@paramlist = @delay_param_names;}

my $isOccupancyData = ($testIndex==1 && ($parameterIndex==1 || $parameterIndex==3)) ? 1 : 0;

my $dataPointer = $parameterIndex;


SWITCH: {
    param("showrawdata") && do {&printPageHeader("Raw Data"); &printRawData($testIndex,$parameterIndex,$chipIndex); last SWITCH;};
    param("getTest") && do {&printPageHeader("Test Data"); &showTestData(); last SWITCH;};
    param("getList") && do{&printPageHeader("Test List");showTestList(); last SWITCH;};

    printPageHeader("Barrel Data");
    &readDaqTable($index-1);
    &readParamTable($testIndex,$parameterIndex,$chipIndex);
    &printBarrelFile($index+2);
    my $sname = "$scriptname\?index=$index\&testIndex=$testIndex\&parameterIndex=$parameterIndex\&chipIndex=$chipIndex\&cellfillindex=$cellfillIndex\&showrawdata=1";
    print "<br><br>&gt;&nbsp;<a href=\"#\" onClick=\"window.open('$sname','RawData','menubar=1,scrollbars=1'); return false;\">View the raw data</a> used to generate this web page.";

}


print qq!
</body>
</html>
!;

sub printRawData() {

    my ($testIndex,$parameterIndex,$chipIndex) = @_;

    my $fname = "$daqLookupTable$testIndexNames[$testIndex]_summary2.txt";


    open(IN,$fname) || printError("Cant open $fname: $!");
    flock(IN,1) || printError("Cant get LOCK_SH on $fname: $!");
    
    print qq!
<html>
<head>
<title>Raw Data</title>
<link rel=stylesheet HREF="http://www.hep.phy.cam.ac.uk/~robinson/styles/davestyles2.css" type="text/css">
!;
print "<\!-- Raw data from file $fname -->";
print qq!
</head>
<body>
!;
   while(<IN>) {
      print "$_ <br>\n";
      }
    close(IN) || printError("Cant close $fname: $!");
    }

sub printError {
 my ($reason) = @_;

   print qq!
  <html>
<head>
<title>Barrel Module Positions</title>
</head>
<body>

<h3><font color=red>Error</font></h3>
<br>
$reason
<br><br>

</body>
</html>
!;
die;
}

sub readParamTable() {

    my ($testIndex,$parameterIndex,$chipIndex) = @_;

    my $fname = "$daqLookupTable$testIndexNames[$testIndex]_summary2.txt";

    open(IN,$fname) || printError("Cant open $fname: $!");
    flock(IN,1) || printError("Cant get LOCK_SH on $fname: $!");
#RUN=1371SCAN=0SN=20220040200007NOISE=M0:1183.21;S1:1211.68;S2:1293.97;S3:1283.25;S4:1257.9;E5:1254.65;M8:1220.48;S9:1280.05;S10:1266.41;S11:1214.65;S12:1338.63;E13:1307.26CFACTORS=M0:1.00;S1:1.00;S2:1.00;S3:1.00;S4:1.00;E5:1.00;M8:1.00;S9:1.00;S10:1.00;S11:1.00;S12:1.00;E13:1.00
    while(<IN>) {
       if(/.*RUN=(\d+)SCAN=(\d+)SN=(\d{14})DATA=(.*)DCS=(.*)DEFECTS=(.*)CFACTORS=(.*)REF=(.*)/) {
            my $runno=$1;
	    my $scanno=$2;
	    my $sn=$3;
	    my $dataline=$4;
	    my $dcsline=$5;
	    my $defectCount=$6;
	    my $cfactorline=$7;
	    my $refline=$8;


	    $datalinks{$sn}="&sn=$sn\&test=$testIndexNames[$testIndex]\&runno=$runno\&scanno=$scanno\&index=$index\&testIndex=$testIndex\&parameterIndex=$parameterIndex\&chipIndex=$chipIndex\&cellfillIndex=$cellfillIndex";

	    my @chipdata = split(/;/,$dataline);
	    my @cfactordata = split(/;/,$cfactorline);
	    my @refdata = split(/;/,$refline);
	    my $chipCount=0;
	    my $chipCount0=0;
	    my $chipCount1=0;
	    my $thisData=0.;
	    my $thisRefData=0.;
	    my $cData=0.;
	    
	    my $defectIndex= ($testIndex==2) ? 2 : 4;
	    my $dcsIndex = ($testIndex==2) ? 3 : 5;
	    my $tdiffIndex = $dcsIndex+8;
	    my $refIndex = ($testIndex==2) ? 1 : 2;

	    if($parameterIndex==$defectIndex) {
	       $thisData=$defectCount;
               $dataMap{$sn}=$thisData;
	       if(exists $daqInfo{$sn}) {$moduleCount++;}
	       next;
	       }
	    if($parameterIndex>=$dcsIndex) {
	       my @thisline= split(/\*/,$dcsline);
	       if($parameterIndex<$tdiffIndex) {$thisData = ($thisline[$parameterIndex-$dcsIndex] eq ".") ? $noDataString : $thisline[$parameterIndex-$dcsIndex];}
	       else {
  	          if($thisline[1] ne "." && $thisline[0] ne ".") {$thisData = abs($thisline[0]-$thisline[1]); $thisData = sprintf("%.2f",$thisData);}
		 else {$thisData = $noDataString;}

		 }
               $dataMap{$sn}=$thisData;
	       if(exists $daqInfo{$sn}) {$moduleCount++;}
	       next;
	       }

	    my $doComparison=0;

	    $dataPointer=$parameterIndex;
	    if($parameterIndex>=$refIndex) {
	       $doComparison=1;
	       $dataPointer = $parameterIndex-$refIndex;
	       }
	       
	    my $scaleReferenceData = !($testIndex==2 || ($testIndex==1 && $dataPointer==1));

            if($chipIndex<=4) {  # mean overall
	        my $total=0.;
		my $totalRef=0.;
		my $minValue=99999.;
		my $maxValueRef=0.;
		my $minValueRef=0.;
		my $maxValue=0.;
		my $mean0=0.;
		my $meanRef0=0.;
		my $mean1=0.;
		my $meanRef1=0.;
	        $thisData=0.;
		$thisRefData=0.;
		$cData=0.;
	        for(my $i=0;$i<12;$i++) {
		   my ($chipName,$dataStar) = split(/:/,$chipdata[$i]);
		   my @datas = split(/\*/,$dataStar);
		   my @reference_datas = split(/\*/,$refdata[$i]);
		   my $data = $datas[$dataPointer];
		   my $reference = ($doComparison) ? $reference_datas[$dataPointer] : 0.;
		   my ($cname,$cdata) = split(/:/,$cfactordata[$i]);
		   if($scaleReferenceData) {$reference *= $cdata;}
		   if($data eq "nan" || $data eq "0") {next;}
		   $total += $data;
		   $totalRef += $reference;
		   if($i<6) {$mean0 += $data; $meanRef0 += $reference; $chipCount0++;}
		   if($i>=6) {$mean1 += $data; $meanRef1 += $reference; $chipCount1++;}
		   if($data<$minValue) {$minValue=$data; $minValueRef=$reference;}
		   if($data>$maxValue) {$maxValue=$data; $maxValueRef=$reference;}
		   $cData += $cdata;
		   $chipCount++;
		   }

		if($chipIndex==0) {
		   $thisData = ($chipCount>0) ? $total/$chipCount : $noDataString;
		   $thisRefData = ($chipCount>0) ? $totalRef/$chipCount : $noDataString;
		   }
		elsif($chipIndex==1) {$thisData = $maxValue; $thisRefData=$maxValueRef;}
		elsif($chipIndex==2) {$thisData = $minValue; $thisRefData=$minValueRef;}
		elsif($chipIndex==3) {
		    $thisData = ($chipCount0>0) ? $mean0/$chipCount0 : $noDataString;
		    $thisRefData = ($chipCount0>0) ? $meanRef0/$chipCount0 : $noDataString;
		    }
		elsif($chipIndex==4) {
		    $thisData = ($chipCount1>0) ? $mean1/$chipCount1 : $noDataString;
		    $thisRefData = ($chipCount1>0) ? $meanRef1/$chipCount1 : $noDataString;
		    }

		if($chipCount>0) {$cData /= $chipCount;}
		}

             else {
	        my ($chipName,$dataStar) = split(/:/,$chipdata[$chipIndex-5]);
		my @datas = split(/\*/,$dataStar);
		$thisData = $datas[$dataPointer];
		my ($cname,$cdata) = split(/:/,$cfactordata[$chipIndex-5]);
		my @reference_datas=split(/\*/,$refdata[$chipIndex-5]);
		if($doComparison) {
		  $thisRefData = ($scaleReferenceData) ? $reference_datas[$dataPointer] * $cdata : $reference_datas[$dataPointer];
		  }
		else {$thisRefData=0.;}
		$cData=$cdata;
		}
	  if($thisData eq "nan" || $thisData==0) {$thisData=$noDataString;}
          if($thisData ne $noDataString) {
	     if($doComparison) {
#	       printf "doing comparison of $thisData - $thisRefData <br>\n";
	       if($thisRefData!=0.) {$thisData-=$thisRefData;}
	       else {$thisData=$noDataString;}
	       }
	     }
          if($thisData ne $noDataString) {$thisData = ($isOccupancyData==1) ? sprintf("%.2E",$thisData) : sprintf("%.2f",$thisData);}

          $dataMap{$sn}=$thisData;
	  if(exists $daqInfo{$sn}) {$moduleCount++;}
	  $cData = sprintf("%.3f",$cData);
	  $cMap{$sn}=$cData;

	  }
        }
     close(IN) || printError("Cant close datafile: $!");
   }




sub readDaqTable() {

      my ($this_index) = @_;
      my $lookup = "${daqLookupTable}moduleMounts$this_index".".txt";
      if(!(-e "$lookup")) {return;}

      open(IN,$lookup) || printError("Cant open $lookup: $!");
      flock(IN,1) || printError("Cant get LOCK_SH on $lookup: $!");
      
      my @lines = <IN>;
      close(IN) || printError("Cant close $lookup: $!");
      push(@lines,"20220380200139=Barrel 3, Row 27, Position 6");

      foreach my $line (@lines) {
        if($line=~/(\d{14})=Barrel\s*(\d+),\s*Row\s*(\d+),\s*Position\s*([-\d]+)/) {
            $daqInfo{$1}="Barrel $2, Row $3, Position $4";
            $barrelCount[$2-3]++;
            my $position = $4;
            $position+=6;
            if($position>=7) {$position--;}
            my $newpos = ($position + ($3 * 12))<<4;
            my $index = $2 + $newpos;
            $daqMapping{$index} = $1;
            $daqInfoExists++;
            }
          }

#      close(IN) || printError("Cant close $lookup: $!");
      }


sub printBarrelFile {

my $barrel=$_[0];
my $barrelIndex=$barrel-3;
my $rows = $barrelRows[$barrelIndex];
#my $thisTitle = "Latest $testNames[$testIndex] $parameterNames[$parameterIndex] Data for Barrel $barrel";

print qq!

<h3>SctRodDaq Barrel $barrel Data Display</h3>
This page is currently displaying the most recent <b>$testIndexNames[$testIndex]</b> data for <b>$moduleCount</b> modules.<br><br>
Data for this webscript is collated automatically over night, so new data may not be visible for up to 24 hours.<br>
Also note that modules which have not been shipped to Oxford in the SCT database will not appear, even if they are physically on the barrel.<br>
For further details of tests etc, click on the run number in the <a href="$logScript">SctRodDaq Logbook</a><br>
Please note that row numbering on this display refers to the DAQ row numbering, which starts from 0.
LMT numbering starts from 1, so row 0 is equivalent to LMT01.

<br><br>

!;


 print qq!
<form method="POST">
<table border=0 cellspacing=1 cellpadding=0>
<col width=250>
<col width=190>
<col width=180>
<col width=180>
<tr><td colspan=4><hr></td></tr>
<tr><td colspan=4 align=left><b>Display Update Options</b></td></tr>
<tr><td colspan=4><br></td></tr>
<tr><td colspan=4 align=left>
!;
my $noTests = @testIndexNames;
printf "Test Type:&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;";
for(my $i=0;$i<$noTests;$i++) {
 if($i>0) {printf "&nbsp;&nbsp;&nbsp;&nbsp;-&nbsp;&nbsp;&nbsp;&nbsp;";}
 printf "<a href=$scriptname\?testIndex=$i\&parameterIndex=0>$testIndexNames[$i]</a>";
 }
print qq!
</td></tr>
<tr><td colspan=4><br></td></tr>
<tr>

<td align=left nowrap>Parameter:&nbsp;&nbsp;<SELECT NAME="parameterIndex">
!;
  my $count=0;
  foreach (@paramlist) {
    my $selectflag = ($parameterIndex == $count) ? "SELECTED" : "";
    print "<option value=$count $selectflag>$_</option>\n";
    $count++;
    }
print qq!
</SELECT></td>

<td align=left nowrap>Chip (if app):&nbsp;&nbsp;<SELECT NAME="chipIndex">
!;
  $count=0;
  foreach (@chipNames) {
    my $selectflag = ($chipIndex == $count) ? "SELECTED" : "";
    print "<option value=$count $selectflag>$_</option>\n";
    $count++;
    }

print qq!
  </SELECT></td>
  <td align=left nowrap>Cell Label:&nbsp;&nbsp;<SELECT NAME="cellfillIndex">
!;
    $count=0;
    foreach (@cellfills) {
      my $selectflag = ($cellfillIndex==$count) ? "SELECTED" : "";
      print "<option value=$count $selectflag>$_</option>\n";
      $count++;
    }
print qq!
</SELECT></td>
<td align=left nowrap><INPUT TYPE="SUBMIT" NAME="viewplots" VALUE="Refresh Display"></td></tr>
<tr><td colspan=4><br></td></tr>
<tr><td colspan=4><hr></td></tr>

</table>
<input type="HIDDEN" name="index" VALUE="$index">
<input type="HIDDEN" name="parameterIndex" VALUE="$parameterIndex">
<input type="HIDDEN" name="testIndex" VALUE="$testIndex">
</form>
<br>
!;


print qq!
<table border=0 cellspacing=5 cellpadding=0>
<col width=150>
<col width=300>
<col width=400>
<tr><td align=right nowrap>Displayed Test:</td><td valign=top align=left nowrap><font color=navy>$testIndexNames[$testIndex] </font></td>
<td rowspan=3 valign=center align=left>
!;
&printColorScale();

printf "</td></tr>";
printf "<tr><td align=right nowrap>Displayed Colour scale:</td><td align=left nowrap> <font color=navy>$paramlist[$parameterIndex]";
if($dataPointer<=1) {printf " ($chipNames[$chipIndex])";}
printf "</font></td></tr>";
printf "<tr><td align=right nowrap>Displayed Cell Label:</td><td align=left nowrap><font color=navy>$cellfills[$cellfillIndex] </font></td></tr>";

printf "</table>";

print qq!
<br>
Hints:<br>
&gt;&nbsp; To view a detailed test report, click on the data value.<br>
&gt;&nbsp; To view all tests on a module, select Cell Label to 'serialNo' and click on the serial number.<br>
<br>

<table border=0 cellspacing=4 cellpadding=0>


<tr>
 <td nowrap width="50" height="0"></td>
 <td nowrap width="90" height="0"></td>
 <td nowrap width="90" height="0"></td>
 <td nowrap width="90" height="0"></td>
 <td nowrap width="90" height="0"></td>
 <td nowrap width="90" height="0"></td>
 <td nowrap width="90" height="0"></td>
 <td nowrap width="10" height="0"></td>
 <td nowrap width="90" height="0"></td>
 <td nowrap width="90" height="0"></td>
 <td nowrap width="90" height="0"></td>
 <td nowrap width="90" height="0"></td>
 <td nowrap width="90" height="0"></td>
 <td nowrap width="90" height="0"></td>
</tr>

<tr><td nowrap></td><td nowrap colspan=13 align=center><b>Module Position</b></td></tr>

<tr>
<td nowrap></td>
<td nowrap align=center><b>-6</b></td>
<td nowrap align=center><b>-5</b></td>
<td nowrap align=center><b>-4</b></td>
<td nowrap align=center><b>-3</b></td>
<td nowrap align=center><b>-2</b></td>
<td nowrap align=center><b>-1</b></td>
<td nowrap></td>
<td nowrap align=center><b>1</b></td>
<td nowrap align=center><b>2</b></td>
<td nowrap align=center><b>3</b></td>
<td nowrap align=center><b>4</b></td>
<td nowrap align=center><b>5</b></td>
<td nowrap align=center><b>6</b></td>
</tr>
!;

for (my $row=0;$row<$rows; $row++) {
 print "<tr><td nowrap align=right><b>Row $row: </b></td>\n";
 for (my $pos=0;$pos<12;$pos++) {
   my $posindex = ($pos + ($row * 12))<<4;
   my $index = $barrel + $posindex;
   my $content = "";
   my $cellColor = "#eeeeff";
   if(exists $daqMapping{$index}) {
     my $sn = $daqMapping{$index};
     if(exists $dataMap{$sn}) {
        if($cellfillIndex==0) {$content="<a href=${scriptname}\?getTest=1\&$datalinks{$sn} class=\"white\">$dataMap{$sn}</a>";}
	elsif($cellfillIndex==1) {$content="<a href=${scriptname}\?getList=1\&$datalinks{$sn} class=\"white\">$sn</a>";}
	elsif($cellfillIndex==2) {$content = (exists $cMap{$sn}) ? $cMap{$sn} : "Unavailable";}
	else {$content="";}
        $cellColor = &getCellColor($dataMap{$sn});
	}
     }

   print "<td nowrap bgcolor=\"$cellColor\" align=center>$content</td>";
   if($pos==5) {print "<td></td>";}
   }
 print "</tr>\n";
 }
 print "</table>\n";



}

sub getCellColor() {


  my ($data) = @_;
  
  if($data eq $noDataString) {return "#f1f1f1";}

  my $color = "000000";
  my $bluebit=0;
  my $redbit=0;
  my $greenbit=0;
  
  my $paramKey = $paramlist[$parameterIndex];
#  if(!(exists colorHashC0{$paramKey})) {printError("No color scale");}

  if($data < $colorHashC0{$paramKey}[0]) {
    $color = sprintf("%02x%02x%02x",$colorHashC0{$paramKey}[1],$colorHashC0{$paramKey}[2],$colorHashC0{$paramKey}[3]);
    return $color;
    }
    
  if($data > $colorHashC3{$paramKey}[0]) {
    $color = sprintf("%02x%02x%02x",$colorHashC3{$paramKey}[1],$colorHashC3{$paramKey}[2],$colorHashC3{$paramKey}[3]);
    return $color;
    }


  my $value0 = $colorHashC0{$paramKey}[0];
  my $value1 = $colorHashC1{$paramKey}[0];
  my $value2 = $colorHashC2{$paramKey}[0];
  my $value3 = $colorHashC3{$paramKey}[0];

  if($data<$value1) {
    my $diff = $data - $value0;
    my $max = $value1 - $value0;
    my $frac = $diff/$max;

    my $redbit0 = $colorHashC0{$paramKey}[1];
    my $greenbit0 = $colorHashC0{$paramKey}[2];
    my $bluebit0 = $colorHashC0{$paramKey}[3];
    
    my $redbit1 = $colorHashC2{$paramKey}[1];
    my $greenbit1 = $colorHashC2{$paramKey}[2];
    my $bluebit1 = $colorHashC2{$paramKey}[3];


    if($redbit0 == $redbit1) {$redbit=$redbit0;}
    elsif($redbit1 > $redbit0) {$redbit = $redbit0 + ($frac * ($redbit1-$redbit0));}
    else {$redbit = $redbit0 - ($frac * ($redbit0-$redbit1));}

    if($greenbit0 == $greenbit1) {$greenbit=$greenbit0;}
    elsif($greenbit1 > $greenbit0) {$greenbit = $greenbit0 + ($frac * ($greenbit1-$greenbit0));}
    else {$greenbit = $greenbit0 - ($frac * ($greenbit0-$greenbit1));}

    if($bluebit0 == $bluebit1) {$bluebit=$bluebit0;}
    elsif($bluebit1 > $bluebit0) {$bluebit = $bluebit0 + ($frac * ($bluebit1-$bluebit0));}
    else {$bluebit = $bluebit0 - ($frac * ($bluebit0-$bluebit1));}

    $color = sprintf("%02x%02x%02x",$redbit,$greenbit,$bluebit);

    return "#$color";

    }
    
  if($data>$value2) {
    my $diff = $data - $value2;
    my $max = $value3 - $value2;
    my $frac = $diff/$max;

    my $redbit0 = $colorHashC2{$paramKey}[1];
    my $greenbit0 = $colorHashC2{$paramKey}[2];
    my $bluebit0 = $colorHashC2{$paramKey}[3];
    
    my $redbit1 = $colorHashC3{$paramKey}[1];
    my $greenbit1 = $colorHashC3{$paramKey}[2];
    my $bluebit1 = $colorHashC3{$paramKey}[3];


    if($redbit0 == $redbit1) {$redbit=$redbit0;}
    elsif($redbit1 > $redbit0) {$redbit = $redbit0 + ($frac * ($redbit1-$redbit0));}
    else {$redbit = $redbit0 - ($frac * ($redbit0-$redbit1));}

    if($greenbit0 == $greenbit1) {$greenbit=$greenbit0;}
    elsif($greenbit1 > $greenbit0) {$greenbit = $greenbit0 + ($frac * ($greenbit1-$greenbit0));}
    else {$greenbit = $greenbit0 - ($frac * ($greenbit0-$greenbit1));}

    if($bluebit0 == $bluebit1) {$bluebit=$bluebit0;}
    elsif($bluebit1 > $bluebit0) {$bluebit = $bluebit0 + ($frac * ($bluebit1-$bluebit0));}
    else {$bluebit = $bluebit0 - ($frac * ($bluebit0-$bluebit1));}

    $color = sprintf("%02x%02x%02x",$redbit,$greenbit,$bluebit);

    return "#$color";

    }
    
  if($data>=$value1 && $data<=$value2) {
    my $diff = $data - $value1;
    my $max = $value2 - $value1;
    my $frac = $diff/$max;

    my $redbit0 = $colorHashC1{$paramKey}[1];
    my $greenbit0 = $colorHashC1{$paramKey}[2];
    my $bluebit0 = $colorHashC1{$paramKey}[3];
    
    my $redbit1 = $colorHashC2{$paramKey}[1];
    my $greenbit1 = $colorHashC2{$paramKey}[2];
    my $bluebit1 = $colorHashC2{$paramKey}[3];


    if($redbit0 == $redbit1) {$redbit=$redbit0;}
    elsif($redbit1 > $redbit0) {$redbit = $redbit0 + ($frac * ($redbit1-$redbit0));}
    else {$redbit = $redbit0 - ($frac * ($redbit0-$redbit1));}

    if($greenbit0 == $greenbit1) {$greenbit=$greenbit0;}
    elsif($greenbit1 > $greenbit0) {$greenbit = $greenbit0 + ($frac * ($greenbit1-$greenbit0));}
    else {$greenbit = $greenbit0 - ($frac * ($greenbit0-$greenbit1));}

    if($bluebit0 == $bluebit1) {$bluebit=$bluebit0;}
    elsif($bluebit1 > $bluebit0) {$bluebit = $bluebit0 + ($frac * ($bluebit1-$bluebit0));}
    else {$bluebit = $bluebit0 - ($frac * ($bluebit0-$bluebit1));}

    $color = sprintf("%02x%02x%02x",$redbit,$greenbit,$bluebit);

    return "#$color";

    }


  return "#ffffff";
  }

sub printColorScale {

my $paramKey = $paramlist[$parameterIndex];
my $C0color = sprintf("#%02x%02x%02x",$colorHashC0{$paramKey}[1],$colorHashC0{$paramKey}[2],$colorHashC0{$paramKey}[3]);
my $C1color = sprintf("#%02x%02x%02x",$colorHashC1{$paramKey}[1],$colorHashC1{$paramKey}[2],$colorHashC1{$paramKey}[3]);
my $C2color = sprintf("#%02x%02x%02x",$colorHashC2{$paramKey}[1],$colorHashC2{$paramKey}[2],$colorHashC2{$paramKey}[3]);
my $C3color = sprintf("#%02x%02x%02x",$colorHashC3{$paramKey}[1],$colorHashC3{$paramKey}[2],$colorHashC3{$paramKey}[3]);
my $label0 = ($isOccupancyData) ? sprintf("%.1E",$colorHashC0{$paramKey}[0]) : $colorHashC0{$paramKey}[0];
my $label1 = ($isOccupancyData) ? sprintf("%.1E",$colorHashC1{$paramKey}[0]) : $colorHashC1{$paramKey}[0];
my $label2 = ($isOccupancyData) ? sprintf("%.1E",$colorHashC2{$paramKey}[0]) : $colorHashC2{$paramKey}[0];
my $label3 = ($isOccupancyData) ? sprintf("%.1E",$colorHashC3{$paramKey}[0]) : $colorHashC3{$paramKey}[0];

print qq!
<table border=0 cellspacing=0 cellpadding=0>
<tr><td align=center valign=bottom>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;${label0}&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;</td>
<td align=center valign=bottom>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;${label1}&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;</td>
<td align=center valign=bottom>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;${label2}&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;</td>
<td align=center valign=bottom>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;${label3}&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;</td></tr>
<tr><td align=center valign=bottom>|</td>
<td align=center valign=bottom>|</td>
<td align=center valign=bottom>|</td>
<td align=center valign=bottom>|</td></tr>
<tr>


<td align=center bgcolor="$C0color">&nbsp;</td>
<td align=center bgcolor="$C1color">&nbsp;</td>
<td align=center bgcolor="$C2color">&nbsp;</td>
<td align=center bgcolor="$C3color">&nbsp;</td></tr>
</table>
!;
}

sub printPageHeader() {
my ($title) = @_;
print qq!
<html>
<head>
<title>$title</title>
<link rel=stylesheet HREF="http://www.hep.phy.cam.ac.uk/~robinson/styles/davestyles2.css" type="text/css">
!;
print "<\!-- \@author Dave Robinson 21/10/2004 -->";
print qq!
</head>
<body>
!;
}

sub showTestData() {
 my $runno = param("runno") ? param("runno") : "";
 my $scanno = param("scanno") ? param("scanno") : "0";
 my $testNameHeader = param("test") ? param("test") : "";
 my $sn = param("sn") ? param("sn") : "";

 if($runno eq "" || $testNameHeader eq "" || $sn eq "") {printError("invalid arguments to script");}

 my $fname = "${summaryDir}${testNameHeader}TestResult.${runno}.${scanno}.${sn}.summary";

 if(!-e $fname) {printError("$fname does not exist.");}

 open(IN,$fname) || printError("Cant open $fname $!");
 flock(IN,1) || printError("Cant flock $fname $!");


 
 my $printedDAQInfo=0;
 my $printedDCSInfo=0;
 my $printedDefectTable=0;
 my $delayData=0;
 my $printedTableData=0;
 
 my $nextLineIsHost=0;
 my $nextLineIsTemperatures=0;
my $nextLineIsHV=0;
my $nextLineIsVCC=0;
my $nextLineIsVDD=0;

my $nextLineIsScanCount=0;
my $nextLineIsScanPoints=0;
my $nextLineIsDelays=0;


 my $passflag="Unknown";
 my $problemflag="Unknown";
 my $date="Unknown";
 my $host="Unknown";
 
 my $chan1="Unknown";
 my $defectname = "Unknown";
 
 my $scanCount=0;
 my @scanPoints=();
 
 my $table=0;
 my $nextLineIsTableData=0;

 print qq!
 <h3>$testNameHeader Test Report for ${sn}, Run $runno, Scan $scanno </h3>

 !;

  my $linkback = "$scriptname\?index=$index\&testIndex=$testIndex\&parameterIndex=$parameterIndex\&chipIndex=$chipIndex\&cellfillindex=$cellfillIndex";
 printf "\&gt;&nbsp;&nbsp;<a href=$linkback>Click here</a> to return to the main display.<br><br>";
 
 $linkback = "$scriptname\?getList=1\&sn=$sn\&index=$index\&testIndex=$testIndex\&parameterIndex=$parameterIndex\&chipIndex=$chipIndex\&cellfillindex=$cellfillIndex";
 printf "\&gt;&nbsp;&nbsp;<a href=$linkback>Click here</a> to list ALL tests for this module.<br><br>";

 while(<IN>) {
   if(/^\s+(.*)/) {$_=$1;}
# printf "$_ <br>\n";
   if(/.*PASSED\s*:\s*(.*)/) {$passflag=$1;}
   if(/.*PROBLEM\s*:\s*(.*)/) {$problemflag=$1;}
   if(/.*TEST[ _]+DATE\s*:\s*(.*)/) {$date=$1;}
   if(/^#HOST.*/) {$nextLineIsHost=1; next;}
   if($nextLineIsHost==1) {
      if(/.*"(.*)"/) {$host=$1;}
      $nextLineIsHost=0;
      }


   if(/[\%0D]+CS_INFO.*/) {
     $printedDAQInfo=1;
     printDAQInfo($host,$date,$passflag,$problemflag);
     print qq!


     <b>DCS Info</b>
     <table border=1>
     <tr><td><b>T0</b></td><td><b>T1</b></td><td><b>TDiff</b></td><td><b>VDet</b></td><td><b>IDet</b></td><td><b>Vcc</b></td><td><b>Icc</b></td><td><b>Vdd</b></td><td><b>Idd</b></td></tr>
     !;
     }

   if(/#T0\s*T1.*/) {$nextLineIsTemperatures=1; next;}
   if(/#VDET\s*IDET.*/) {$nextLineIsHV=1; next;}
   if(/#VCC\s*ICC.*/) {$nextLineIsVCC=1; next;}
   if(/#VDD\s*IDD.*/) {$nextLineIsVDD=1; next;}

   if($nextLineIsTemperatures==1) {
     my ($t0,$t1) = split(/\s+/);

     printf "<tr><td>". sprintf("%.1f",$t0) ."</td><td>" . sprintf("%.1f",$t1) . "</td>";
     my $tdiff="Unknown";
     if($t0 ne "." && $t1 ne ".") {$tdiff = sprintf("%.1f",abs($t0-$t1));}
     printf "<td>$tdiff</td>";
     $nextLineIsTemperatures=0;
     }
    if($nextLineIsHV==1) {
     my ($d0,$d1) = split(/\s+/);
     printf "<td>$d0</td><td>$d1</td>";
     $nextLineIsHV=0;
     }
    if($nextLineIsVCC==1) {
     my ($d0,$d1) = split(/\s+/);
     printf "<td>$d0</td><td>$d1</td>";
     $nextLineIsVCC=0;
     }
    if($nextLineIsVDD==1) {
     my ($d0,$d1) = split(/\s+/);
     printf "<td>$d0</td><td>$d1</td></tr></table><br>";
     $printedDCSInfo=1;
     $nextLineIsVDD=0;
     }

   if(/^#N_POINTS.*/) {$nextLineIsScanCount=1; next;}
   if(/^#POINTS.*/) {$nextLineIsScanPoints=1; next;}

   if($nextLineIsScanCount) {
     $scanCount=$_;
     $nextLineIsScanCount=0;
     }
   if($nextLineIsScanPoints) {
     if(/^#.*/) {printScanData($scanCount,@scanPoints); $nextLineIsScanPoints=0;}
     my @p = split(/\s+/);
     foreach my $point (@p) {
        if($point ne ".") {push(@scanPoints,$point);}
	}
     }
     
   if(/^#LoopA.*/) {
      $table=1;
      if($printedDAQInfo==0) {printDAQInfo($host,$date,$passflag,$problemflag); $printedDAQInfo=1;}
      if($printedDCSInfo==0) {printDCSInfo(); $printedDCSInfo=1;}
      print qq!
      <b>RC Fits</b>
      <table border=1>
      <tr><td><b>Chip</b></td><td><b>Func</b></td><td><b>P0</b></td><td><b>P1</b></td><td><b>P2</b></td></tr>
      !;
      }
   elsif(/^#LoopB.*/) {
      $table=1;
      if($printedDAQInfo==0) {printDAQInfo($host,$date,$passflag,$problemflag); $printedDAQInfo=1;}
      if($printedDCSInfo==0) {printDCSInfo(); $printedDCSInfo=1;}
      print qq!
      <b>RC Data</b>
      <table border=1>
      <tr><td><b>Chip</b></td><td><b>vt50</b></td><td><b>Vt50 RMS</b></td><td><b>Gain</b></td><td><b>Gain RMS</b></td><td><b>Offset</b></td><td><b>Offset RMS</b></td><td><b>Output Noise</b></td><td><b>ENC input Noise</b></td><td><b>ENC noise RMS</b></td></tr>
      !;
      }
   elsif(/^#\s*Offset\s*MeanOcc\s*RMSOcc\s*EstENC.*/) {
      $table=1;
      if($printedDAQInfo==0) {printDAQInfo($host,$date,$passflag,$problemflag); $printedDAQInfo=1;}
      if($printedDCSInfo==0) {printDCSInfo(); $printedDCSInfo=1;}
      print qq!
      <b>Occupancy Data</b>
      <table border=1>
      <tr><td><b>Chip</b></td><td><b>Offset</b></td><td><b>Occupancy</b></td><td><b>Occupancy RMS</b></td><td><b>ENC Noise</b></td></tr>
      !;
      }
   if(/^#([MSE]+\d{1,2})$/) {
      $nextLineIsTableData=1;
      printf "<tr><td nowrap>$1</td>";
      next;
      }
   if($table && /^#$/) {
      $table=0;
      $printedTableData++;
      printf "</table><br>";
      }
   if($nextLineIsTableData) {
     my @dataline = split(/\s+/);
     foreach my $databit (@dataline) {
       printf "<td nowrap>${databit}&nbsp;&nbsp;&nbsp;</td>";
       }
     printf "</tr>";
     $nextLineIsTableData=0;
     }
     
   if(/#DELAY/) {
      if($printedDAQInfo==0) {printDAQInfo($host,$date,$passflag,$problemflag); $printedDAQInfo=1;}
      if($printedDCSInfo==0) {printDCSInfo(); $printedDCSInfo=1;}
      print qq!
      <b>Delay Data</b>
      <table border=1>
      <tr><td><b>Chip</b></td><td><b>Delay</b></td></tr>
      !;
      $delayData=1;
      next;
      }
   if(/^#M0\s+S1\s+S2\s+S3\s+S4\s+E5/) {$nextLineIsDelays=1; next;}
   if(/^#M8\s+S9\s+S10\s+S11\s+S12\s+E13/) {$nextLineIsDelays=7; next;}
   if($nextLineIsDelays) {
     my @delays = split(/\s+/);
     my $ycount=4+$nextLineIsDelays;
     foreach my $delay (@delays) {
        printf "<tr><td>$chipNames[$ycount]&nbsp;&nbsp;</td><td>$delay&nbsp;&nbsp;</td></tr>\n";
	$ycount++;
	}
     if($nextLineIsDelays==7) {printf "</table><br>"; $printedTableData++;}
     $nextLineIsDelays=0;
     }

   if(/DEFECT\s+NAME\s*:\s*(.*)/) {
      if($printedDefectTable==0) {printDefectTable(); $printedDefectTable=1;}
      $defectname=$1;
      }
   if(/FIRST\s+CHANNEL\s*:\s*(\d+)/) {$chan1=$1;}
   if(/LAST\s+CHANNEL\s*:\s*(\d+)/) {
     my $chans = ($chan1 eq $1) ? $chan1 : $chan1 . " - " . $1;
     printf "<tr><td>$chans &nbsp;&nbsp;</td><td>$defectname</td></tr>\n";
     }
   if(/#(\d+\s*defects\s*found\s*affecting\s*(\d+)\s*strips)/) {
      printf "</table><br>";
      printf "&gt;&nbsp;&nbsp;$1 <br>\n";
      }
      
   if(/#No\s*defects\s*found/) {
     print qq!
       <b>Defects</b><br>
       None found.
       <br><br>
       !;
       }






   }
 close(IN) || printError("Cant close $fname $!");
 
 if($printedTableData==0) {
       if($printedDAQInfo==0) {printDAQInfo($host,$date,$passflag,$problemflag); $printedDAQInfo=1;}
      if($printedDCSInfo==0) {printDCSInfo(); $printedDCSInfo=1;}
      print qq!
      <b>Test Data</b><br>
      Sorry, test data reports are not available yet for this type of test.
      !;
      }



 
 }
 sub printDefectTable() {
   print qq!
   <b>Defects</b>
   <table border=1>
   <tr><td><b>Channel(s)</b></td><td><b>Defect</b></td></tr>
   !;
   }
 sub printDAQInfo() {
   my ($host,$date,$passflag,$problemflag)=@_;
   print qq!
    <b>DAQ info</b>
     <table border=1>
     <tr><td><b>Host</b></td><td><b>Date</b></td><td><b>Passed?</b></td><td><b>Problem?</b></td></tr>
     <tr><td>$host</td><td>$date</td><td>$passflag</td><td>$problemflag</td></tr>
     </table>
     <br>
     !;
     }
 sub printScanData() {
   my ($pointcount,@points)=@_;
   print qq!
    <b>Scan Info</b>
     <table border=1>
     <tr><td><b>#Scans</b></td><td><b>Points</b></td></tr>
     <tr><td>$pointcount</td><td>
     !;
     my $count=0;
     foreach my $p (@points) {
       if($p ne ".") {
          if($count>0) {printf ", ";}
	  $count++;
	  printf "$p";
	  }
	}
    print qq!
     </td></tr>
     </table>
     <br>
     !;
     }
 sub printDCSInfo() {
    print qq!
      <b>DCS Info</b><br>
      Not available.
      <br><br>
      !;
      }

sub showTestList() {
 my $sn = param("sn") ? param("sn") : "";

 if($sn eq "") {printError("invalid arguments to script");}

 my %fileHash=();

 opendir(HDATA,$summaryDir) || bailout("Cant open directory $summaryDir: $!");

 while(my $fname = readdir(HDATA)) {
   if($fname=~/(.*)TestResult.(\d+).(\d+).${sn}.summary/) {
      my $tname=$1;
      my $runno=$2;
      my $scanno=$3;
      my $link = "<a href=$scriptname\?getTest=1\&sn=$sn\&test=$tname\&runno=$runno\&scanno=$scanno\&index=$index\&testIndex=$testIndex\&parameterIndex=$parameterIndex\&chipIndex=$chipIndex\&cellfillIndex=$cellfillIndex>$tname</a>";
      my $key = $scanno + ($runno * 1000000);
      $fileHash{$key}=$link;
      }
   }
 closedir (HDATA);

 printf "<h3>Tests of $sn </h3>";
 
 my $linkback = "$scriptname\?index=$index\&testIndex=$testIndex\&parameterIndex=$parameterIndex\&chipIndex=$chipIndex\&cellfillIndex=$cellfillIndex";
 printf "\&gt;&nbsp;&nbsp;<a href=$linkback>Click here</a> to return to the display.<br><br>";
  print qq!
 <table border=0 cellspacing=0 cellpadding=0>
 <col width=180>
 <col width=100>
 <col width=100>
 <tr><td><b>Test</b></td><td><b>Run</b></td><td><b>Scan</b></td></tr>
 !;
 my $lineCount=0;
 foreach my $key (sort keys %fileHash) {
  my $runno = $key/1000000;
  $runno =sprintf("%d",$runno);
  my $scanno = $key%1000000;
  my $background = ($lineCount%2==0) ? "bgcolor=\"#f6f6f6\"" : "bgcolor=\"#eeeeff\"";
   print "<tr $background><td>$fileHash{$key}</td><td>$runno</td><td>$scanno</td></tr>\n";
   $lineCount++;
   }
  print "</table><br>";
  
  }
